#ifndef _SYS_AUD_H
#define _SYS_AUD_H

#include <wchar.h>
#include <parsec/parsec_aud.h>

#ifdef __cplusplus
extern "C" {
#endif
/* examples */

/*	
	aud_entry_t	entry;
	aud = aud_init(AUD_TYPE_PS);
	aud_create_entry(aud, &entry);
	aud_set_successed(entry, 1);
	aud_set_failed(entry, 2); 
	aud_set_pid(pid, aud);
	aud_free(aud);
*/

#define AUD_TYPE_PROC			0x0
#define AUD_TYPE_FILE			0x1
#define AUD_TYPE_FILE_DEFAULT		0x3

#define AUD_FILE_ANY			FILE_AUDIT_ANY
#define AUD_FILE_UID			FILE_AUDIT_UID
#define AUD_FILE_GID			FILE_AUDIT_GID


	struct __aud;
	struct __aud_entry;

	typedef struct __aud		*aud_t;
	typedef struct __aud_entry	*aud_entry_t;

	typedef uint8_t 	aud_entry_type_t;
	typedef uint32_t 	aud_entry_flags_t;
	typedef uint32_t	aud_entry_id_t;

	extern aud_t	aud_init(int type);
	extern aud_t	aud_dup(aud_t);
	extern int	aud_copy(aud_t aud, aud_t src);
    extern int	aud_cmp(aud_t a, aud_t b);

    extern int	aud_valid(aud_t);

	extern aud_t 	aud_copy_int(const void *buf);
	extern ssize_t	aud_copy_ext(void *buf, aud_t aud, size_t size);

#define AUD_FMT_NUM		0
#define AUD_FMT_LONG		1
#define AUD_FMT_TXT		0x100

	extern char *	aud_to_text(aud_t aud, ssize_t *size, int flags);
	extern wchar_t *aud_to_wtext(aud_t mac, ssize_t *size, int flags);
	
	extern int	aud_from_text(aud_t aud, const char *text);
	extern ssize_t	aud_size(aud_t aud);

	extern int	aud_create_entry(aud_t aud, aud_entry_t *); /* or add empty one */

	extern int 	aud_set_type(aud_entry_t, aud_entry_type_t type); /* only for file */
	extern int 	aud_set_id(aud_entry_t, aud_entry_id_t id); /* only for file */
	extern int 	aud_set_successed(aud_entry_t, aud_entry_flags_t successed);
	extern int	aud_set_failed(aud_entry_t, aud_entry_flags_t failed);

	extern aud_entry_type_t aud_get_type(aud_entry_t);
	extern aud_entry_id_t 	aud_get_id(aud_entry_t);
	extern aud_entry_flags_t aud_get_successed(aud_entry_t);
	extern aud_entry_flags_t aud_get_failed(aud_entry_t);

	extern int	aud_delete_entry(aud_entry_t); /* delete entry */
	extern int	aud_copy_entry(aud_entry_t dst, aud_entry_t str); /* copy one */
/* enumerator */	
	extern int	aud_type(aud_t);
	extern aud_entry_t	aud_next_entry(aud_t, aud_entry_t entry);
	extern int	aud_entries(aud_t); /* num of entries */	
/* file */
	extern aud_t	aud_get_file(const char *path);
	extern aud_t	aud_get_file_default(const char *path);

	extern int	aud_set_file(const char *path, aud_t);
	extern aud_t	aud_get_fd(int fd);
	extern aud_t	aud_get_fd_default(int fd);
	extern int	aud_set_fd(int fd, aud_t);
/* ps */
	extern int	aud_set_pid(pid_t pid, aud_t);
	aud_t		aud_get_pid(pid_t);

	static inline int aud_set_proc(aud_t aud) { return aud_set_pid(0, aud); }
	static inline aud_t aud_get_proc(void) { return aud_get_pid(0); }
	
	extern void 	aud_free(aud_t aud);
	
#ifdef __cplusplus
}
#endif
#endif				/* _SYS_AUD_H */
