/* $Id: aud_db.h 482 2007-11-20 15:36:55Z pkun $ */

#ifndef _PARSEC_AUD_DB_H_
#define _PARSEC_AUD_DB_H_

#include <stdio.h>		/* For FILE */
#include <sys/types.h>		/* For pid_t */

#include <parsec/parsec.h>
#include <parsec/parsec_aud.h>

#ifdef __cplusplus
extern "C" {
#endif

/***************************************************
 *           WORKING WITH CONFIG FILE              *
 ***************************************************/

#define AUDIT_TYPE_USER   0x01
#define AUDIT_TYPE_GROUP  0x02
#define AUDIT_TYPE_OTHER  0x04
#define AUDIT_TYPE_ANY    (AUDIT_TYPE_USER | AUDIT_TYPE_GROUP | AUDIT_TYPE_OTHER)

#define AUDIT_KEYWORD_USER  "user"
#define AUDIT_KEYWORD_GROUP "group"
#define AUDIT_KEYWORD_OTHER "other"

	typedef uint32_t audit_ps_t;

	struct audit_entry {
		int type;	/* The entry type: AUDIT_ENTRY_USER, .._GROUP, or .._OTHER. */
		char *name;	/* User's or group's name. For "other" entry this field is empty. */
		audit_ps_t succ;	/* Audit these events in case of success. */
		audit_ps_t fail;	/* Audit these events in case of failure. */
	};

	struct __aud_han_t;
	typedef struct __aud_han_t *aud_han_t;

	extern struct audit_entry *parse_aud(const char *buff, 
		struct audit_entry *aud);
	extern char *compose_aud(const struct audit_entry *aud);

	// thread-safe base API
	extern aud_han_t openauddb_r(void);
	extern int closeauddb_r(aud_han_t han);

	extern void freeaudent_r(struct audit_entry *aud);
	extern int setaudent_r(aud_han_t han, int for_types);
	extern int endaudent_r(aud_han_t han);
	extern struct audit_entry *getaudent_r(aud_han_t han);
	extern int putaudent_r(aud_han_t han, 
		const struct audit_entry *data);
	extern struct audit_entry *getaudtype_r(aud_han_t han, 
		int type, const char *name);
	extern int rmaudtype_r(aud_han_t han, int type, const char *name);
	
	extern struct audit_entry *fgetaudent_r(FILE * fd);
	extern int fputaudent_r(FILE * fd, const struct audit_entry *data);

	// old common API
	extern int setaudent(int for_types);
	extern void endaudent(void);
	extern struct audit_entry *getaudent(void);
	extern struct audit_entry *fgetaudent(FILE * f);
	extern void fputaudent(FILE * f, const struct audit_entry *data);
	extern int putaudent(const struct audit_entry *data);
	extern struct audit_entry *getaudtype(int type, const char *name);
	extern int rmaudtype(int type, const char *name);
	
#ifdef __cplusplus
}
#endif

#endif	/* _PARSEC_AUD_DB_H_ */
