/* $Id: parsec_aud.h 561 2008-06-17 11:41:06Z pkun $ */

#ifndef _PARSEC_AUD_H_
#define _PARSEC_AUD_H_

#include <sys/types.h>

#include <parsec/parsec.h>

/* Process audit */
#define PS_LOG_OPEN	(1<<0)
#define PS_LOG_CREATE	(1<<1)
#define PS_LOG_EXEC	(1<<2)
#define PS_LOG_DELETE	(1<<3)
#define PS_LOG_CHMOD	(1<<4)
#define PS_LOG_CHOWN	(1<<5)
#define PS_LOG_MOUNT	(1<<6)
#define PS_LOG_MODULE	(1<<7)
#define PS_LOG_UID	(1<<8)
#define PS_LOG_GID	(1<<9)
#define PS_LOG_AUDIT	(1<<10)
#define PS_LOG_XATTR	(1<<11)
#define PS_LOG_MAC	(1<<12)
#define PS_LOG_CAP	(1<<13)
#define PS_LOG_CHROOT	(1<<14)
#define PS_LOG_RENAME	(1<<15)
#define PS_LOG_NET	(1<<16)

/* File audit */
#define FS_LOG_OPEN	(1<<0)
#define FS_LOG_EXEC	(1<<1)
#define FS_LOG_DELETE	(1<<2)
#define FS_LOG_CHMOD	(1<<3)
#define FS_LOG_CHOWN	(1<<4)
#define FS_LOG_AUDIT	(1<<5)
#define FS_LOG_XATTR	(1<<6)
#define FS_LOG_MAC	(1<<7)
#define FS_LOG_CREATE	(1<<8) /* create in dir */
#define FS_LOG_MODIFY	(1<<9)

/* types */
#define FILE_AUDIT_REGULAR	0x0
#define FILE_AUDIT_DEFAULT	0x1
#define FILE_AUDIT_TYPES	0x2

#define MAX_FILE_AUDIT_USER_SIZE 4096

#define FILE_AUDIT_ANY 1
#define FILE_AUDIT_UID 2
#define FILE_AUDIT_GID 3

#define FILE_AUDIT_NONE    0x00
#define FILE_AUDIT_UNKNOWN 0x0f

#define FILE_AUDIT_TYPE  0x0f
#define FILE_AUDIT_FLAG  ~(FILE_AUDIT_TYPE)

#ifdef __cplusplus
extern "C" {
#endif

/***************************************************
 *           WORKING WITH PROCESSES                *
 ***************************************************/
	typedef struct {
		uint32_t successed;
		uint32_t failed;
	} parsec_audit_t;

	extern int parsec_setaud(pid_t pid, const parsec_audit_t *flags);
	extern int parsec_getaud(pid_t pid, parsec_audit_t *flags);

/***************************************************
 *           WORKING WITH FILE SYSTEM              *
 ***************************************************/
typedef struct _parsec_file_audit_item {
	uint32_t successed;
	uint32_t failed;
	uint8_t  type;
	uint32_t id;
} parsec_file_audit_item_t;

/* warning ! do not redefine this for compat layer safety P.K.*/

typedef struct _parsec_file_audit {
	uint8_t type;
	uint8_t flags;		// reserved
	uint16_t num;
	parsec_file_audit_item_t ids[0];
} parsec_file_audit_t;

	extern int parsec_chaud(const char *fname,
			      const parsec_file_audit_t *data, size_t size);
	extern int parsec_fchaud(int fd, const parsec_file_audit_t *data,
			       size_t size);
	extern int parsec_stataud(const char *fname, parsec_file_audit_t *data,
				size_t *size);
	extern int parsec_fstataud(int fd, parsec_file_audit_t *data, size_t *size);
	
#ifdef __cplusplus
}
#endif
#endif				/* _PARSEC_AUD_H */
